@@ -116,7 +116,8 @@ def download_network_cached(cache_url, dir_url, software_url, software_root,
 @fallback_call
 def upload_network_cached(software_root, software_url, cached_key,
     cache_url, dir_url, path, logger, signature_private_key_file,
-    shacache_cert_file, shacache_key_file, shadir_cert_file, shadir_key_file):
+    shacache_ca_file, shacache_cert_file, shacache_key_file,
+    shadir_ca_file, shadir_cert_file, shadir_key_file):
     """Upload file to a network cache server"""
     if not LIBNETWORKCACHE_ENABLED:
         return False
@@ -141,10 +142,14 @@ def upload_network_cached(software_root, software_url, cached_key,
     # convert '' into None in order to call nc nicely
     if not signature_private_key_file:
         signature_private_key_file = None
+    if not shacache_ca_file:
+        shacache_ca_file = None
     if not shacache_cert_file:
         shacache_cert_file = None
     if not shacache_key_file:
         shacache_key_file = None
+    if not shadir_ca_file:
+        shadir_ca_file = None
     if not shadir_cert_file:
         shadir_cert_file = None
     if not shadir_key_file:
@@ -152,8 +157,10 @@ def upload_network_cached(software_root, software_url, cached_key,
     try:
         nc = NetworkcacheClient(cache_url, dir_url,
             signature_private_key_file=signature_private_key_file,
+            shacache_ca_file=shacache_ca_file,
             shacache_cert_file=shacache_cert_file,
             shacache_key_file=shacache_key_file,
+            shadir_ca_file=shadir_ca_file,
             shadir_cert_file=shadir_cert_file,
             shadir_key_file=shadir_key_file)
     except TypeError:
