@@ -30,12 +30,12 @@
 
 from lxml import etree
 import random
-import sqlite3
 import string
 from slapos.slap.slap import Computer, ComputerPartition, \
     SoftwareRelease, SoftwareInstance, NotFoundError
 from slapos.proxy.db_version import DB_VERSION
 import slapos.slap
+from slapos.util import sqlite_connect
 
 from flask import g, Flask, request, abort
 import xml_marshaller
@@ -50,10 +50,19 @@ class UnauthorizedError(Exception):
   pass
 
 
+# cast everything to string, utf-8 encoded
+def to_str(v):
+  if isinstance(v, str):
+    return v
+  if not isinstance(v, unicode):
+    v = unicode(v)
+  return v.encode('utf-8')
+
+
 def xml2dict(xml):
   result_dict = {}
   if xml is not None and xml != '':
-    tree = etree.fromstring(xml.encode('utf-8'))
+    tree = etree.fromstring(to_str(xml))
     for element in tree.iter(tag=etree.Element):
       if element.tag == 'parameter':
         key = element.get('id')
@@ -70,7 +79,7 @@ def dict2xml(dictionary):
   instance = etree.Element('instance')
   for parameter_id, parameter_value in dictionary.iteritems():
     # cast everything to string
-    parameter_value = str(parameter_value)
+    parameter_value = unicode(parameter_value)
     etree.SubElement(instance, "parameter",
                      attrib={'id': parameter_id}).text = parameter_value
   return etree.tostring(instance,
@@ -136,7 +145,7 @@ def execute_db(table, query, args=(), one=False, db_version=None, log=False, db=
 
 
 def connect_db():
-  return sqlite3.connect(app.config['DATABASE_URI'])
+  return sqlite_connect(app.config['DATABASE_URI'])
 
 def _getTableList():
   return g.db.execute("SELECT name FROM sqlite_master WHERE type='table' ORDER BY Name").fetchall()
@@ -538,6 +547,19 @@ def getAllocatedSlaveInstance(slave_reference, requested_computer_id):
   # XXX: check there is only one result
   return execute_db(table, q, args, one=True)
 
+def getRootPartition(reference):
+  p = 'SELECT * FROM %s WHERE reference=?'
+  parent_partition = execute_db('partition', p, [reference], one=True)
+
+  while parent_partition is not None:
+    parent_reference = parent_partition['requested_by']
+    if not parent_reference or parent_reference == reference:
+      break
+    reference = parent_reference
+    parent_partition = execute_db('partition', p, [reference], one=True)
+
+  return parent_partition
+
 def requestNotSlave(software_release, software_type, partition_reference, partition_id, partition_parameter_kw, filter_kw, requested_state):
   instance_xml = dict2xml(partition_parameter_kw)
   requested_computer_id = filter_kw['computer_guid']
@@ -554,10 +576,6 @@ def requestNotSlave(software_release, software_type, partition_reference, partit
   a = args.append
   q = 'UPDATE %s SET slap_state="busy"'
 
-  if requested_state:
-    q += ', requested_state=?'
-    a(requested_state)
-
   if partition is None:
     partition = execute_db('partition',
         'SELECT * FROM %s WHERE slap_state="free" and computer_reference=?',
@@ -575,6 +593,21 @@ def requestNotSlave(software_release, software_type, partition_reference, partit
       a(partition_id)
     if not software_type:
       software_type = 'RootSoftwareInstance'
+  else:
+    # XXX Check if software_release should be updated
+    if partition['software_release'].encode() != software_release:
+      q += ' ,software_release=?'
+      a(software_release)
+    if partition['requested_by']:
+      root_partition = getRootPartition(partition['requested_by'])
+      if root_partition and root_partition['requested_state'] != "started":
+        # propagate parent state to child
+        # child can be stopped or destroyed while parent is started
+        requested_state = root_partition['requested_state']
+
+  if requested_state:
+    q += ', requested_state=?'
+    a(requested_state)
 
   #
   # XXX change software_type when requested
@@ -732,6 +765,10 @@ def softwareInstanceRename():
 def getComputerPartitionStatus():
   return xml_marshaller.xml_marshaller.dumps('Not implemented.')
 
+@app.route('/computerBang', methods=['POST'])
+def computerBang():
+  return xml_marshaller.xml_marshaller.dumps('')
+
 @app.route('/getComputerPartitionCertificate', methods=['GET'])
 def getComputerPartitionCertificate():
   # proxy does not use partition certificate, but client calls this.
