@@ -331,6 +331,20 @@ class IComputerPartition(IBuildoutController, IRequester):
       text -- message log of the status
     """
 
+  def getFullHostingIpAddressList():
+    """
+    Returns a dictionnary containing the latest status of the
+    computer partition.
+    """
+
+  def setComputerPartitionRelatedInstanceList(instance_reference_list):
+    """
+    Set relation between this Instance and all his children.
+
+    instance_reference_list -- list of instances requested by this Computer
+                               Partition.   
+    """
+
 class IComputer(Interface):
   """
   Computer interface specification
