@@ -36,10 +36,12 @@ __all__ = ["slap", "ComputerPartition", "Computer", "SoftwareRelease",
            "Supply", "OpenOrder", "NotFoundError",
            "ResourceNotReady", "ServerError", "ConnectionError"]
 
+import os
 import json
 import logging
 import re
 import urlparse
+import hashlib
 from util import xml2dict
 
 import netaddr
@@ -66,6 +68,7 @@ fallback_logger.addHandler(fallback_handler)
 
 
 DEFAULT_SOFTWARE_TYPE = 'RootSoftwareInstance'
+COMPUTER_PARTITION_REQUEST_LIST_TEMPLATE_FILENAME = '.slapos-request-transaction-%s'
 
 class SlapDocument:
   def __init__(self, connection_helper=None, hateoas_navigator=None):
@@ -80,6 +83,7 @@ class SlapRequester(SlapDocument):
   """
   Abstract class that allow to factor method for subclasses that use "request()"
   """
+
   def _requestComputerPartition(self, request_dict):
     try:
       xml = self._connection_helper.POST('requestComputerPartition', data=request_dict)
@@ -183,7 +187,7 @@ class SoftwareProductCollection(object):
     self.get = self.__getattr__
 
   def __getattr__(self, software_product):
-      self.logger.info('Getting best Software Release corresponging to '
+      self.logger.info('Getting best Software Release corresponding to '
                        'this Software Product...')
       software_release_list = \
           self.slap.getSoftwareReleaseListFromSoftwareProduct(software_product)
@@ -405,9 +409,38 @@ class ComputerPartition(SlapRequester):
     self._partition_id = partition_id
     self._request_dict = request_dict
 
+    # Just create an empty file (for nothing requested yet)
+    self._updateTransactionFile(partition_reference=None)
+
   def __getinitargs__(self):
     return (self._computer_id, self._partition_id, )
 
+  def _updateTransactionFile(self, partition_reference=None):
+    """
+    Store reference to all Instances requested by this Computer Parition
+    """
+    # Environ variable set by Slapgrid while processing this partition
+    instance_root = os.environ.get('SLAPGRID_INSTANCE_ROOT', '')
+    if not instance_root or not self._partition_id:
+      return
+
+    transaction_file_name = COMPUTER_PARTITION_REQUEST_LIST_TEMPLATE_FILENAME % self._partition_id
+    transaction_file_path = os.path.join(instance_root, self._partition_id,
+                                        transaction_file_name)
+
+    try:
+      if partition_reference is None:
+        if os.access(os.path.join(instance_root, self._partition_id), os.W_OK):
+          if os.path.exists(transaction_file_path):
+            return
+          transac_file = open(transaction_file_path, 'w')
+          transac_file.close()
+      else:
+        with open(transaction_file_path, 'a') as transac_file:
+          transac_file.write('%s\n' % partition_reference)
+    except OSError:
+      return
+
   def request(self, software_release, software_type, partition_reference,
               shared=False, partition_parameter_kw=None, filter_kw=None,
               state=None):
@@ -439,6 +472,7 @@ class ComputerPartition(SlapRequester):
         'filter_xml': xml_marshaller.dumps(filter_kw),
         'state': xml_marshaller.dumps(state),
     }
+    self._updateTransactionFile(partition_reference)
     return self._requestComputerPartition(request_dict)
 
   def building(self):
@@ -564,8 +598,26 @@ class ComputerPartition(SlapRequester):
       return self._software_release_document
 
   def setConnectionDict(self, connection_dict, slave_reference=None):
-    if self.getConnectionParameterDict() != connection_dict:
-      self._connection_helper.POST('setComputerPartitionConnectionXml', data={
+    if self.getConnectionParameterDict() == connection_dict:
+      return
+
+    if slave_reference is not None:
+      # check the connection parameters from the slave
+
+      # Should we check existence?
+      slave_parameter_list = self.getInstanceParameter("slave_instance_list")
+      slave_connection_dict = {}
+      for slave_parameter_dict in slave_parameter_list:
+        if slave_parameter_dict.get("slave_reference") == slave_reference:
+          connection_parameter_hash = slave_parameter_dict.get("connection-parameter-hash", None)
+          break
+
+      # Skip as nothing changed for the slave
+      if connection_parameter_hash is not None and \
+        connection_parameter_hash == hashlib.sha256(str(connection_dict)).hexdigest():
+        return
+
+    self._connection_helper.POST('setComputerPartitionConnectionXml', data={
           'computer_id': self._computer_id,
           'computer_partition_id': self._partition_id,
           'connection_xml': xml_marshaller.dumps(connection_dict),
@@ -606,6 +658,24 @@ class ComputerPartition(SlapRequester):
                 }
             )
     return xml_marshaller.loads(xml)
+  
+  def getFullHostingIpAddressList(self):
+    xml = self._connection_helper.GET('getHostingSubscriptionIpList',
+            params={
+                'computer_id': self._computer_id,
+                'computer_partition_id': self._partition_id,
+                }
+            )
+    return xml_marshaller.loads(xml)
+
+  def setComputerPartitionRelatedInstanceList(self, instance_reference_list):
+    self._connection_helper.POST('updateComputerPartitionRelatedInstanceList',
+        data={
+          'computer_id': self._computer_id,
+          'computer_partition_id': self._partition_id,
+          'instance_reference_xml': xml_marshaller.dumps(instance_reference_list)
+          }
+        )
 
 def _addIpv6Brackets(url):
   # if master_url contains an ipv6 without bracket, add it
@@ -1009,4 +1079,3 @@ class SlapHateoasNavigator(HateoasNavigator):
     instance_url = self.hateoasGetLinkFromLinks(instance_list, reference)
     instance = self._hateoasGetInformation(instance_url)
     return instance
-
